"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _lodash = require("lodash");

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class Accessories {
  constructor() {
    this.comparator = this.comparator.bind(this);
    this._accessories = {};
    this._ignored = {};
  }

  getAccessoryKey(accessory) {
    const context = accessory.context || accessory;
    return `${context.object_type}/${context.object_id}`;
  }

  get(device) {
    const key = this.getAccessoryKey(device);
    return this._accessories[key];
  }

  ignore(device) {
    const key = this.getAccessoryKey(device);
    if (this._ignored[key]) {
      return false;
    }

    this._ignored[key] = device;
    return true;
  }

  add(accessory) {
    const key = this.getAccessoryKey(accessory);
    return this._accessories[key] = accessory;
  }

  remove(accessory) {
    const key = this.getAccessoryKey(accessory);
    const _accessory = this._accessories[key];
    delete this._accessories[key];
    return _accessory;
  }

  forEach(fn) {
    return _lodash2.default.forEach(this._accessories, fn);
  }

  intersection(devices) {
    const accessories = _lodash2.default.values(this._accessories);
    return _lodash2.default.intersectionWith(devices, accessories, this.comparator);
  }

  diffAdd(devices) {
    const accessories = _lodash2.default.values(this._accessories);
    return _lodash2.default.differenceWith(devices, accessories, this.comparator);
  }

  diffRemove(devices) {
    const accessories = _lodash2.default.values(this._accessories);
    return _lodash2.default.differenceWith(accessories, devices, this.comparator);
  }

  comparator(accessory1, accessory2) {
    return this.getAccessoryKey(accessory1) === this.getAccessoryKey(accessory2);
  }
}
exports.default = Accessories;
module.exports = exports["default"];
//# sourceMappingURL=Accessories.js.map