"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.platformName = exports.pluginName = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _child_process = require("child_process");

var _child_process2 = _interopRequireDefault(_child_process);

var _fs = require("fs");

var _fs2 = _interopRequireDefault(_fs);

var _lodash = require("lodash");

var _lodash2 = _interopRequireDefault(_lodash);

var _compareVersions = require("compare-versions");

var _compareVersions2 = _interopRequireDefault(_compareVersions);

var _joi = require("joi");

var _joi2 = _interopRequireDefault(_joi);

var _Accessories = require("./Accessories");

var _Accessories2 = _interopRequireDefault(_Accessories);

var _AccessoryHelper = require("./AccessoryHelper");

var _AccessoryHelper2 = _interopRequireDefault(_AccessoryHelper);

var _configSchema = require("./configSchema");

var _configSchema2 = _interopRequireDefault(_configSchema);

var _devices = require("./devices");

var _devices2 = _interopRequireDefault(_devices);

var _Subscriptions = require("./Subscriptions");

var _Subscriptions2 = _interopRequireDefault(_Subscriptions);

var _WinkClient = require("./WinkClient");

var _WinkClient2 = _interopRequireDefault(_WinkClient);

var _package = require("../package.json");

var _package2 = _interopRequireDefault(_package);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const pluginName = exports.pluginName = "homebridge-wink";
const platformName = exports.platformName = "Wink";

class WinkPlatform {
  constructor(log, config, api) {
    if (!config) {
      log("Plugin not configured.");
      return;
    }

    const result = _joi2.default.validate(config, _configSchema2.default);
    if (result.error) {
      log.error("Invalid config.", result.error.message);
      return;
    }

    this.log = log;
    this.config = this.cleanConfig(config);
    this.api = api;

    this.checkVersion();

    this.definitions = (0, _devices2.default)(api.hap);
    this.accessories = new _Accessories2.default();
    this.accessoryHelper = new _AccessoryHelper2.default({
      config: this.config,
      definitions: this.definitions,
      hap: api.hap,
      log,
      onChange: this.handleAccessoryStateChange.bind(this)
    });
    this.client = new _WinkClient2.default({
      config: this.config,
      log,
      updateConfig: config => this.updateConfig(config)
    });
    this.interval = null;
    this.subscriptions = new _Subscriptions2.default();

    this.subscriptions.on("device-list", () => this.refreshDevices());
    this.subscriptions.on("device-update", device => {
      this.log(`Received update notification: ${device.name} (${device.object_type}/${device.object_id})`);
      this.updateDevice(device);
    });
    this.subscriptions.on("unknown-message", message => {
      this.log.warn("Received unknown notification:", message);
    });

    this.api.on("didFinishLaunching", this.didFinishLaunching.bind(this));
  }

  updateConfig(newConfig) {
    const configPath = this.api.user.configPath();
    const file = _fs2.default.readFileSync(configPath);
    const config = JSON.parse(file);
    const platConfig = config.platforms.find(x => x.name == this.config.name);
    _lodash2.default.extend(platConfig, newConfig);
    const serializedConfig = JSON.stringify(config, null, "  ");
    _fs2.default.writeFileSync(configPath, serializedConfig, "utf8");
    _lodash2.default.extend(this.config, newConfig);
  }

  checkVersion() {
    _child_process2.default.exec(`npm view ${_package2.default.name} version`, (error, stdout) => {
      const latestVersion = stdout && stdout.trim();
      if (latestVersion && (0, _compareVersions2.default)(stdout.trim(), _package2.default.version) > 0) {
        this.log.warn(`NOTICE: New version of ${_package2.default.name} available: ${latestVersion}`);
      }
    });
  }

  cleanConfig(config) {
    const newConfig = _extends({
      debug: false,
      direct_access: true,
      fan_ids: [],
      hide_groups: [],
      hide_ids: [],
      outlet_ids: [],
      switch_ids: [],
      window_ids: []
    }, config);

    ["fan_ids", "hide_ids", "window_ids"].forEach(field => {
      newConfig[field] = newConfig[field].map(id => id.toString());
    });

    return newConfig;
  }

  handleAccessoryStateChange(accessory, state) {
    return this.client.updateDevice(accessory, state);
  }

  configureAccessory(accessory) {
    if (!this.accessories) {
      return;
    }

    this.patchAccessory(accessory);
    this.accessories.add(accessory);
    this.log(`Loaded from cache: ${accessory.context.name} (${accessory.context.object_type}/${accessory.context.object_id})`);
  }

  patchAccessory(accessory, device) {
    if (device) {
      accessory.context = device;
    }
    accessory.definition = this.accessoryHelper.getDefinition(accessory.context);
    Object.defineProperty(accessory, "merged_state", {
      get: function get() {
        return _extends({}, this.context.last_reading, this.context.desired_state);
      }
    });
  }

  didFinishLaunching() {
    var _this = this;

    return _asyncToGenerator(function* () {
      const authenticated = yield _this.client.authenticate();

      if (authenticated) {
        _this.accessories.forEach(function (accessory) {
          const device = accessory.context;
          const newAccessory = _this.getNewAccessory(device);

          _this.accessoryHelper.configureAccessory(accessory);
          _this.accessoryHelper.removeDeprecatedServices(accessory, newAccessory);
        });

        _this.interval = setInterval(function () {
          return _this.refreshDevices();
        }, 60 * 60 * 1000);

        _this.refreshDevices();
      }
    })();
  }

  getNewAccessory(device) {
    const uuid = this.api.hap.uuid;

    const deviceId = uuid.isValid(device.uuid) ? device.uuid : uuid.generate(device.uuid);
    const accessory = new this.api.platformAccessory(device.name, deviceId);
    this.patchAccessory(accessory, device);
    this.accessoryHelper.configureAccessory(accessory);
    return accessory;
  }

  addDevice(device) {
    const accessory = this.getNewAccessory(device);
    this.api.registerPlatformAccessories(pluginName, platformName, [accessory]);
    this.accessories.add(accessory);
    this.log(`Added: ${accessory.context.name} (${accessory.context.object_type}/${accessory.context.object_id})`);
  }

  updateDevice(device) {
    const accessory = this.accessories.get(device);
    this.accessoryHelper.updateAccessoryState(accessory, device);
    this.subscriptions.subscribe(device.subscription);
  }

  refreshDevices() {
    var _this2 = this;

    return _asyncToGenerator(function* () {
      try {
        _this2.log("Refreshing devices...");

        // Request user - this should ensure that pubnub subscriptions don't stop working
        yield _this2.client.getUser();

        const response = yield _this2.client.getDevices();

        var _sortDevices = _this2.sortDevices(response.data),
            _sortDevices2 = _slicedToArray(_sortDevices, 3);

        const hubs = _sortDevices2[0],
              devices = _sortDevices2[1],
              ignoreDevices = _sortDevices2[2];


        _this2.client.processHubs(hubs);

        const toRemove = _this2.accessories.diffRemove(devices);
        const toUpdate = _this2.accessories.intersection(devices);
        const toAdd = _this2.accessories.diffAdd(devices);

        if (response.subscription) {
          _this2.subscriptions.subscribe(response.subscription);
        }

        toRemove.forEach(_this2.removeAccessory, _this2);
        toUpdate.forEach(_this2.updateDevice, _this2);
        toAdd.forEach(_this2.addDevice, _this2);
        ignoreDevices.forEach(_this2.ignoreDevice, _this2);

        _this2.log("Devices refreshed");
      } catch (e) {
        _this2.log.error("Failed to refresh devices.", e);
      }
    })();
  }

  sortDevices(devices) {
    const supportedDevices = [];
    const ignoreDevices = [];

    const currentDevices = devices.filter(device => !device.hidden_at);

    currentDevices.filter(device => device.object_type !== "hub").forEach(device => {
      const definition = this.definitions[device.object_type];

      if (!definition) {
        ignoreDevices.push([device, "Not supported by HomeKit"]);
        return;
      }

      const hide_groups = this.config.hide_groups.indexOf(definition.group) !== -1 || this.config.hide_groups.indexOf(definition.type) !== -1;

      if (hide_groups) {
        ignoreDevices.push([device, "Hidden by hide_groups config option"]);
        return;
      }

      const hide_ids = this.config.hide_ids.indexOf(device.object_id) !== -1;

      if (hide_ids) {
        ignoreDevices.push([device, "Hidden by hide_ids config option"]);
        return;
      }

      supportedDevices.push(device);
    });

    const hubIds = supportedDevices.filter(x => x.hub_id).map(x => x.hub_id);

    const hubs = currentDevices.filter(device => device.object_type === "hub" && hubIds.includes(device.hub_id) && device.last_reading.ip_address);

    return [hubs, supportedDevices, ignoreDevices];
  }

  ignoreDevice(data) {
    var _data = _slicedToArray(data, 2);

    const device = _data[0],
          reason = _data[1];

    if (!this.accessories.ignore(device)) {
      return;
    }

    this.log(`${reason}: ${device.name} (${device.object_type}/${device.object_id})`);
  }

  removeAccessory(accessory) {
    if (this.accessories.remove(accessory)) {
      this.api.unregisterPlatformAccessories(pluginName, platformName, [accessory]);
      this.log(`Removed: ${accessory.context.name} (${accessory.context.object_type}/${accessory.context.object_id})`);
    }
  }
}
exports.default = WinkPlatform;
//# sourceMappingURL=WinkPlatform.js.map